import json
import hashlib
import logging
import requests
from .allin_utils import getRandomStr, createSign

_log = logging.getLogger()

# 交易接口
_QUERY_URL = 'https://vsp.allinpay.com/apiweb/unitorder/query'

class AllinQuery(object):

    @staticmethod
    def DebugAllinQuery():
        ''' 测试用的接口
        '''
        return AllinQuery('100581048160005', '990440148166000', '00000003', 'a0ea3fa20dbd7bb4d5abf1d59d63bae8')

    def __init__(self, orgid, cusid, appid, md5Key):
        ''' 查询接口
        :param orgid: 机构id
        :param cusid: 商户id
        :param appid: 应用id
        :param md5Key: 签名所用的key
        '''
        self.values = {}
        self.values['orgid'] = orgid
        self.values['cusid'] = cusid
        self.values['appid'] = appid
        self.md5Key = md5Key
        self.values['version'] = '11'
    
    def setReqsn(self, reqsn):
        ''' 交易流水 reqsn和trxid必填其一;建议:商户如果同时拥有oldtrxid和oldreqsn,优先使用oldtrxid 
        '''
        self.values['reqsn'] = reqsn
        return self
    
    def setTrxid(self, trxid):
        ''' 交易流水 reqsn和trxid必填其一;建议:商户如果同时拥有oldtrxid和oldreqsn,优先使用oldtrxid 
        '''
        self.values['trxid'] = trxid
        return self

    def query(self, **kw):
        ''' 查询
        :param **kw: 其他支付信息
        '''
        self.values['randomstr'] = getRandomStr()
        self.values.update(kw)

        self.values['sign'] = createSign(self.values, self.md5Key)
        if self._checkValues():
            res = self._post()
            if self._checkValues(res):
                return res
            else:
                _log.error('用户请求查询被拦截，返回结果：%s' % res)
                return '返回参数校验不通过，可能是请求被恶意拦截!'
        else:
            return '参数不合法，请检查请求参数！'
    
    def _checkValues(self, values = None):
        ''' 检查参数是否合法
        :param values:如果为空检查请求参数，否则检查返回参数
        '''
        if values:
            if values['retcode'] == 'SUCCESS':
                v = values.copy()
                sign = v['sign']
                v.pop('sign')
                sign2 = createSign(v, self.md5Key)
                if sign2 == sign:
                    return True
                else:
                    return False
        else:
            if all (k in self.values for k in ('orgid', 'cusid', 'appid', 'randomstr', 'sign')):
                return True
            
        return False

    def _post(self):
        ''' 发送post请求获取二维码
        '''
        r = requests.post(_QUERY_URL, self.values)
        text = json.loads(r.text)
        _log.info('用户发起退款请求，请求参数：%s，请求结果：%s' % (self.values, text))
        return text

# if __name__ == "__main__":
#     allinQuery = AllinQuery.DebugAllinQuery().setTrxid('111994120000928109')
#     res = allinQuery.query()
#     print(res)


import datetime
import hashlib
from random import Random 

_chars = 'AaBbCcDdEeFfGgHhIiJjKkLlMmNnOoPpQqRrSsTtUuVvWwXxYyZz0123456789'
def getDatetime(h = -8):
    ''' 获取日期时间，格式 Wed, 04 Sep 2019 01:40:18 GMT
    :param h: 时差，默认减8小时
    '''
    return (datetime.datetime.now() + datetime.timedelta(hours=h)).strftime("%a, %d %b %Y %H:%M:%S GMT")   

def getDate():
    ''' 获取当前日期,格式 1970-01-01
    '''
    return datetime.datetime.now().strftime("%Y-%m-%d") 

def getRandomStr(length = 10):
    ''' 获取随机字符串
    '''
    salt = ''
    len_chars = len(_chars) - 1  
    random = Random()  
    for i in range(length):  
        # 每次从chars中随机取一位
        salt += _chars[random.randint(0, len_chars)]  
    return salt

def createSign(values, md5Key):
    ''' 生成签名
    '''
    # 把key加进去
    values.update({'key': md5Key})
    # 字典序
    sortArr = sorted(values.items(), key = lambda item : item[0])
    signStr = ''
    # 拼接
    for i in sortArr:
        if i[1] != '':
            signStr += i[0] + '=' + i[1] + '&'
    signStr = signStr.strip('&')
    # MD5
    m2 = hashlib.md5()   
    m2.update(signStr.encode('utf-8'))
    # 去掉key
    values.pop('key')
    return m2.hexdigest().upper()


import json
import logging
import requests
from .allin_utils import getRandomStr, createSign

_log = logging.getLogger()

# 接口地址
_AUTH_CODE_TO_USERID_URL = 'https://vsp.allinpay.com/apiweb/unitorder/authcodetouserid'

class AllinAuthCode(object):

    @staticmethod
    def DebugAllinAuthCode():
        ''' 测试用的支付接口
        '''
        return AllinAuthCode('100581048160005', '990440148166000', '00000003', 'a0ea3fa20dbd7bb4d5abf1d59d63bae8')

    def __init__(self, orgid, cusid, appid, md5Key):
        ''' 统一支付接口
        :param orgid: 机构id
        :param cusid: 商户id
        :param appid: 应用id
        :param md5Key: 签名所用的key
        '''
        self.values = {}
        self.values['orgid'] = orgid
        self.values['cusid'] = cusid
        self.values['appid'] = appid
        self.md5Key = md5Key
        self.values['version'] = '11'

    def setAuthType(self, authtype):
        ''' 授权码类型
        '''
        self.values['authtype'] = authtype
        return self
    
    def setSubAppid(self, sub_appid):
        ''' 微信支付appid
        '''
        self.values['sub_appid'] = sub_appid
        return self

    def authcode(self, authcode, **kw):
        ''' 获取用户ID
        '''
        self.values['randomstr'] = getRandomStr()
        self.values['authcode'] = authcode
        self.values.update(kw)

        self.values['sign'] = createSign(self.values, self.md5Key)
        if self._checkValues():
            res = self._post()
            if self._checkValues(res):
                return res
            else:
                _log.error('用户请求支付被拦截，返回结果：%s' % res)
                return '返回参数校验不通过，可能是请求被恶意拦截!'
        else:
            return '参数不合法，请检查请求参数！'
    
    def _checkValues(self, values = None):
        ''' 检查参数是否合法
        :param values:如果为空检查请求参数，否则检查返回参数
        '''
        if values:
            if values['retcode'] == 'SUCCESS':
                v = values.copy()
                sign = v['sign']
                v.pop('sign')
                sign2 = createSign(v, self.md5Key)
                if sign2 == sign:
                    return True
                else:
                    return False
        else:
            if all (k in self.values for k in ('orgid', 'cusid', 'appid', 'authcode', 'authtype', 'randomstr', 'sign')):
                return True
            
        return False

    def _post(self):
        ''' 发送post请求
        '''
        r = requests.post(_AUTH_CODE_TO_USERID_URL, self.values)
        text = json.loads(r.text)
        _log.info('用户发起支付请求，请求参数：%s，请求结果：%s' % (self.values, text))
        return text


__version__ = '0.0.1'

from .allin_cance import AllinCancel
from .allin_pay import AllinPay
from .allin_query import AllinQuery
from .allin_refund import AllinRefund
from .allin_authcodetouserid import AllinAuthCode
from .allin_wxfacepayinfo import AllinWXFacePay





#!/usr/bin/env python
#-*- coding:utf-8 -*-

#############################################
# File Name: setup.py
# Author: lijin
# Mail: lijin@dingtalk.com
# Created Time:  2019-09-05
#############################################


from setuptools import setup, find_packages		# 没有这个库的可以通过pip install setuptools导入

setup(
    name = "allinpay",
    version = "0.0.1",
    keywords = ("pip", "pathtool","timetool", "magetool", "mage"),							
    description = "通联支付python工具",
    long_description = "通联支付python工具",
    license = "MIT Licence",

    url = "https://github.com/l616769490/allinapy",
    author = "lijin",
    author_email = "lijin@dingtalk.com",

    packages = find_packages(),											# 导入目录下的所有__init__.py包
    include_package_data = True,
    platforms = "any",
    install_requires = []
)


import json
import hashlib
import logging
import requests
from .allin_utils import getRandomStr, createSign

_log = logging.getLogger()

# 统一支付接口地址
_PAY_URL = 'https://vsp.allinpay.com/apiweb/unitorder/pay'

class AllinPay(object):

    @staticmethod
    def DebugAllinPay():
        ''' 测试用的支付接口
        '''
        return AllinPay('100581048160005', '990440148166000', '00000003', 'a0ea3fa20dbd7bb4d5abf1d59d63bae8')

    def __init__(self, orgid, cusid, appid, md5Key):
        ''' 统一支付接口
        :param orgid: 机构id
        :param cusid: 商户id
        :param appid: 应用id
        :param md5Key: 签名所用的key
        '''
        self.values = {}
        self.values['orgid'] = orgid
        self.values['cusid'] = cusid
        self.values['appid'] = appid
        self.md5Key = md5Key
        self.values['version'] = '11'
        self.values['paytype'] = 'A01'
        self.values['validtime'] = '15'
    
    def setPaytype(self, paytype):
        ''' 设置设置支付方式
        :param paytype: https://aipboss.allinpay.com/know/devhelp/main.php?pid=24
        '''
        self.values['paytype'] = paytype
        return self

    def setBody(self, body):
        ''' 订单标题
        '''
        self.values['body'] = body
        return self
    
    def setRemark(self, remark):
        ''' 设置备注信息
        '''
        self.values['remark'] = remark
        return self

    def noCredit(self):
        ''' 指定不能使用信用卡支付
        '''
        self.values['no_credit'] = 'no_credit'
        return self
    
    def setAcct(self, acct):
        ''' 微信支付-用户的微信openid; 支付宝支付-用户user_id; 微信小程序-用户小程序的openid
            小程序支付必填
        '''
        self.values['acct'] = acct
        return self
    
    def setNotifyUrl(self, notifyUrl):
        ''' 交易结果的回调地址
            支付宝支付，小程序支付必填
        '''
        self.values['notify_url'] = notifyUrl
        return self
    
    def setSubAppid(self, subAppid):
        ''' 交易APPID
            小程序支付必填
        '''
        self.values['sub_appid'] = subAppid
        return self

    def pay(self, money, reqsn, **kw):
        ''' 支付
        :param money: 支付金额，单位分
        :param reqsn: 订单号，全局唯一
        :param **kw: 其他支付信息
        '''
        self.values['randomstr'] = getRandomStr()
        self.values['trxamt'] = money
        self.values['reqsn'] = reqsn
        self.values.update(kw)

        self.values['sign'] = createSign(self.values, self.md5Key)
        if self._checkValues():
            res = self._post()
            if self._checkValues(res):
                return res
            else:
                _log.error('用户请求支付被拦截，返回结果：%s' % res)
                return '返回参数校验不通过，可能是请求被恶意拦截!'
        else:
            return '参数不合法，请检查请求参数！'
    
    def _checkValues(self, values = None):
        ''' 检查参数是否合法
        :param values:如果为空检查请求参数，否则检查返回参数
        '''
        if values:
            if values['retcode'] == 'SUCCESS':
                v = values.copy()
                sign = v['sign']
                v.pop('sign')
                sign2 = createSign(v, self.md5Key)
                if sign2 == sign:
                    return True
                else:
                    return False
        else:
            if all (k in self.values for k in ('orgid', 'cusid', 'appid', 'reqsn', 'trxamt', 'reqsn', 'paytype', 'randomstr', 'sign')):
                if self.values['paytype'] == 'A01':
                    if 'notify_url' in self.values:
                        return True
                    else:
                        return False
                if self.values['paytype'] == 'W06':
                    if 'acct' in self.values and 'notify_url' in self.values and 'sub_appid' in self.values:
                        return True
                    else:
                        return False
            
        return True

    def _post(self):
        ''' 发送post请求获取二维码
        '''
        r = requests.post(_PAY_URL, self.values)
        text = json.loads(r.text)
        _log.info('用户发起支付请求，请求参数：%s，请求结果：%s' % (self.values, text))
        return text

# if __name__ == "__main__":
#     allinpay = AllinPay.DebugAllinPay().setBody('支付信息').setNotifyUrl('').setRemark('备注信息')
#     res = allinpay.pay('1', '201909050004')
#     print(res)

#     import qrcode
#     img = qrcode.make(res['payinfo'])
#     img.save('D:\\aa.png')
